//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * スプライトレンダラヘッダ
 * @author Junpee
 */

#ifndef SPRITE_RENDERER_H_
#define SPRITE_RENDERER_H_

#include <Core/Container/ArrayList.h>

namespace Lamp{

class SpriteRequest;
class SpriteRenderState;
class SpriteGraphicsBuffer;

//------------------------------------------------------------------------------
/**
 * スプライトレンダラ
 */
class SpriteRenderer{
public:
	//--------------------------------------------------------------------------
	// 生成、破棄
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 * @param maxPriority 最大優先度
	 * @param defaultPriority デフォルト優先度
	 */
	SpriteRenderer(int maxPriority = 16, int defaultPriority = 8);

	/**
	 * デストラクタ
	 */
	virtual ~SpriteRenderer();

	//--------------------------------------------------------------------------
	// リクエスト
	//--------------------------------------------------------------------------
	/**
	 * リクエスト
	 * @param spriteRequest スプライトリクエスト
	 */
	virtual void request(SpriteRequest* spriteRequest){
		request(spriteRequest, defaultPriority_);
	}

	/**
	 * リクエスト
	 * @param spriteRequest スプライトリクエスト
	 * @param priority 優先度
	 */
	virtual void request(SpriteRequest* spriteRequest, int priority);

	//--------------------------------------------------------------------------
	/**
	 * リクエスト
	 * @param spriteRequest スプライトリクエスト
	 */
	virtual void request(const SpriteRequest* spriteRequest){
		// インターフェース内部で最適化を行うためにconstをはずす
		// 外向けの変数自体はレンダリング時にいじらない
		request((SpriteRequest*)spriteRequest);
	}

	/**
	 * リクエスト
	 * @param spriteRequest スプライトリクエスト
	 * @param priority 優先度
	 */
	virtual void request(const SpriteRequest* spriteRequest, int priority){
		// インターフェース内部で最適化を行うためにconstをはずす
		// 外向けの変数自体はレンダリング時にいじらない
		request((SpriteRequest*)spriteRequest, priority);
	}

	//--------------------------------------------------------------------------
	/**
	 * デフォルトステートのリクエスト
	 */
	virtual void requestDefaultState(){ requestDefaultState(defaultPriority_); }

	/**
	 * デフォルトステートのリクエスト
	 * @param priority 優先度
	 */
	virtual void requestDefaultState(int priority);

	//--------------------------------------------------------------------------
	// レンダリング
	//--------------------------------------------------------------------------
	/**
	 * レンダリング
	 */
	virtual void render(){ render(0, (maxPriority_ - 1)); }

	/**
	 * レンダリング
	 * @param startPriority 開始優先度
	 * @param endPriority 終了優先度
	 */
	virtual void render(int startPriority, int endPriority);

	//--------------------------------------------------------------------------
	// 優先度
	//--------------------------------------------------------------------------
	/**
	 * 最大優先度の取得
	 */
	virtual int getMaxPriority() const{ return maxPriority_; }

	/**
	 * デフォルト優先度の設定
	 * @param defaultPriority デフォルト優先度
	 */
	virtual void setDefaultPriority(int defaultPriority){
		Assert((defaultPriority >= 0) && (defaultPriority < maxPriority_));
		defaultPriority_ = defaultPriority;
	}

	/**
	 * デフォルト優先度の取得
	 * @return デフォルト優先度
	 */
	virtual int getDefaultPriority() const{ return defaultPriority_; }

protected:
	//--------------------------------------------------------------------------
	/**
	 * デフォルトステートの適用
	 */
	virtual void applyDefaultState(SpriteRenderState* renderState);

	/**
	 * レンダラステートの適用
	 */
	virtual void applyRendererState();

private:
	//--------------------------------------------------------------------------
	// コピーコンストラクタの隠蔽
	SpriteRenderer(const SpriteRenderer& copy);

	// 代入コピーの隠蔽
	void operator =(const SpriteRenderer& copy);

	//--------------------------------------------------------------------------
	// リクエスト
	ArrayList<SpriteRequest*>* requests_;
	// グラフィックスバッファ
	SpriteGraphicsBuffer* graphicsBuffer_;
	// 最大優先度
	int maxPriority_;
	// デフォルト優先度
	int defaultPriority_;

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of SPRITE_RENDERER_H_
//------------------------------------------------------------------------------
